@@ -6,21 +6,32 @@
 #[cfg(feature = "51")]
 use crate::pac::GPIO as P0;
 
-#[cfg(not(feature = "51"))]
+#[cfg(feature = "9160")]
+use crate::pac::P0_NS as P0;
+
+#[cfg(not(any(feature = "51", feature = "9160")))]
 use crate::pac::P0;
 
 #[cfg(any(feature = "52833", feature = "52840"))]
 use crate::pac::P1;
 
+use crate::gpio::{
+    Floating, Input, Level, OpenDrain, Output, Pin, Port, PullDown, PullUp, PushPull,
+};
+
+#[cfg(not(feature = "9160"))]
 use {
-    crate::gpio::{
-        Floating, Input, Level, OpenDrain, Output, Pin, Port, PullDown, PullUp, PushPull,
-    },
     crate::pac::gpiote::{EVENTS_IN, EVENTS_PORT, TASKS_OUT},
     crate::pac::GPIOTE,
 };
 
-#[cfg(not(feature = "51"))]
+#[cfg(feature = "9160")]
+use {
+    crate::pac::gpiote0_s::{EVENTS_IN, EVENTS_PORT, TASKS_CLR, TASKS_OUT, TASKS_SET},
+    crate::pac::GPIOTE1_NS as GPIOTE,
+};
+
+#[cfg(not(any(feature = "51", feature = "9160")))]
 use crate::pac::gpiote::{TASKS_CLR, TASKS_SET};
 
 #[cfg(not(feature = "51"))]
